from os.path import join, abspath, dirname

from pathlib import Path

from dateutil import parser

from src.facebook_event_aggregator.scraper.driver import setup_driver
from src.facebook_event_aggregator.scraper.scrape_event_page import scrape_event_page
from src.facebook_event_aggregator.scraper.scrape_events_page import scrape_events_page
from src.facebook_event_aggregator.scraper.scrape_all import scrape_all


def test_scrape_event_page():
    return
    location, image_url = scrape_event_page("file://" + abspath(join(dirname(__file__).replace("\\", "/"), "event_page.htm")))
    assert location == "Le Botanique"
    assert image_url == "https://scontent-bru2-1.xx.fbcdn.net/v/t39.30808-6/384365949_704986814996124_1745966508573235950_n.jpg?stp=dst-jpg_s960x960&_nc_cat=106&ccb=1-7&_nc_sid=5f2048&_nc_ohc=YrHGPpoEZssAX8FBBH7&_nc_oc=AQkB4iDsJKhbaZq2oCPLR0ui77bF_fqimfD7hV3pTDczYXSTcO1tqLKSYFY4YbV-q0I&_nc_ht=scontent-bru2-1.xx&oh=00_AfA6LQP9omvSzhV9fQH8_NVYL8pT2d7rkxnBlOGt87251A&oe=653DDCD4"


def events_page_tester(tmp_path: Path, func, events_as_list:bool=False):
    events_page = "file://" + abspath(join(dirname(__file__).replace("\\", "/"), "events_page.htm"))
    if events_as_list:
        events_page = [events_page]
    events = func(setup_driver(headless=True), events_page, str(tmp_path))
    
    things_to_find = [
        {
            # TODO finish datetime tests
            "name": "Business As Usual #2 Money Talks pt 2 in AMOR",
            #"datetime": parser.parse("25 oct 08:00 +2")



        },
        {
            "name": "Coach Party / Trix - HiFive Concert - UITVERKOCHT!"
        },
        {
            "name": "Eric Steckel / Trix"
        },
        {
            "name": "Brass Against / Trix"
        },
        {
            "name": "It It Anita + Godcaster / Trix"
        },
        {
            "name": "Birds in Row + Walfang + Quentin Sauvé / Trix"
        },
        {
            "name": "Vrijdag Vrijdag met Fabulae Dramatis + C I M E + Wendung"
        },
        {
            "name": "Protomartyr + Es + dust / Trix"
        }
    ]

    for event in events:

        assert event.source == "Trix"
        to_find = things_to_find.pop(0)
        assert event.name == to_find["name"]
        #assert event.location == "Trix" or event.location == "AMOR van De Roma"
        #assert event.location == "Trix"  # TODO: location is not consistent
        print(event.name)
        print(event.source)
        print(event.datetime)
        
        print(event.location)
        print("---")
    


def test_scrape_events_page(tmp_path: Path):
    events_page_tester(tmp_path, scrape_events_page)

def test_scrape_all(tmp_path: Path):
    events_page_tester(tmp_path, scrape_all, events_as_list=True)


from pathlib import Path

from src.facebook_event_aggregator.export.utils import cleanup_images
from src.facebook_event_aggregator.Event import Event
from .helpers import example

def assert_exist(path: Path, filenames, exist=True):
    for filename in filenames:
        assert (path.joinpath(filename)).exists() == exist

def test_cleanup_images(tmp_path: Path):
    events = example["events"]
    
    filenames = ["test.png", events[0].uid + ".png", "meow.png", events[1].uid]

    for filename in filenames:
        with open(str(tmp_path.joinpath(filename)), "w", encoding="UTF-8") as file:
            file.write("")
    
    assert_exist(tmp_path, filenames)

    for file in tmp_path.glob("*"):
        print(file)
    
    cleanup_images(events, str(tmp_path))

    for file in tmp_path.glob("*"):
        print(file)


    assert_exist(tmp_path, [filenames[0], filenames[2]], exist=False)
    assert_exist(tmp_path, [filenames[1], filenames[3]], exist=True)
    
        


from time import sleep

from .driver import setup_driver
from ..utils.url_converter import facebook_www_to_locale

from selenium.webdriver.common.by import By

# Parse the event page in a secondary driver. This is universal for pages & communities
def scrape_event_page(event_url):
    location = None
    image_url = None
    event_url = facebook_www_to_locale(event_url)
    try:
        print("Searching for location & image in " + event_url)
        tmp_driver = setup_driver(headless=True)
        tmp_driver.get(event_url)
        sleep(20)
        info_rows = tmp_driver.find_elements(By.XPATH, "//div/div[1]/div/div[3]/div/div/div/div[1]/div[1]/div[2]/div/div/div[2]/div/div/div/div[1]/div[1]/div/div/div/*")
        # Assume the first row is location, except if it contains specific text
        for info_row in info_rows:
            info = info_row.text
            info_lower = info.lower()
            if "details" in info_lower or "event by" in info_lower or "people respon" in info_lower:
                continue
                    
            # If location hasn't been found yet, assume the first line (that isn't excluded) is
            location = info
            break
            """ The following code can be used to begin implementing duration
            if not location:
                location = info
            if "duration" in info_lower:
                pass  
            """
                
        image_url = tmp_driver.find_element(By.CSS_SELECTOR,"img[data-imgperflogname=\"profileCoverPhoto\"]").get_attribute("src")

        tmp_driver.quit()
                
    except Exception as e:
        print(e)
        print("Location could not be fetched")

    
    return (location, image_url)




from pathlib import Path

from src.facebook_event_aggregator.Event import Event
from src.facebook_event_aggregator.export.to_html import events_to_html

from .helpers import example, check_event_contents_in

def test_events_to_html(tmp_path: Path):
    events_to_html(
        events=example["events"],
        absolute_img_dir=tmp_path.joinpath("images"),
        relative_from_html_img_dir="images/",
        page_urls=example["page_urls"],
        output_dir=tmp_path,
        title=example["title"],
        host_domain=example["host_domain"],
        timezone=example["timezone"]
    )

    with open(tmp_path.joinpath("index.html"), encoding="UTF-8") as file:
        data = file.read()
        for item in example.values():
            if type(item) == list:
                for value in item:
                    if not isinstance(value, Event):
                        assert value in data
                    else:
                        check_event_contents_in(value, data)
            else:
                assert item in data



from pathlib import Path

from ics import Calendar

from src.facebook_event_aggregator.export.to_ics import events_to_ics, _read_ics_if_exists

from .helpers import example, check_event_contents_in, write_test_ics

def test_read_ics_if_exists(tmp_path: Path):

    test_path = tmp_path.joinpath("test.ics")
    
    assert not test_path.exists()
    read_when_not_exist = _read_ics_if_exists(str(test_path))

    assert read_when_not_exist == Calendar()  # Empty calendar
    assert len(read_when_not_exist.events) == 0

    write_test_ics(test_path)
    read_when_exists = _read_ics_if_exists(str(test_path))
    
    assert test_path.exists()
    assert read_when_exists != Calendar()
    assert len(read_when_exists.events) > 0
    



def test_events_to_ics(tmp_path: Path):
    events_to_ics(example["events"], str(tmp_path))

    ical_dir = tmp_path.joinpath("ical/")

    all_filepath = ical_dir.joinpath("all.ics")

    assert ical_dir.joinpath("all.ics").exists()
    with open(str(all_filepath), "r", encoding="utf-8") as all_file:
        all_content = all_file.read()

    for event in example["events"]:
        check_event_contents_in(event, all_content)

        source_filepath = ical_dir.joinpath(event.source.lower() + ".ics")
        assert source_filepath.exists()
        with open(str(source_filepath), "r", encoding="UTF-8") as source_file:
            source_contents = source_file.read()
            check_event_contents_in(event, source_contents)
        



"""An Event """

# Built-in imports
from copy import deepcopy
from json import load, dump
from os.path import join, basename
from glob import glob
from re import match, RegexFlag

# Package imports
from datetime import timedelta, datetime
from dateutil import parser
from slugify import slugify

# Local imports
from .utils.url_converter import facebook_locale_to_www

class Event(object):
    """
    name = str
    datetime = date
    url = str
    """


    def __init__(self, name: str, datetime_param: str, source: str, location: str , url: str):
        self.name = name
        # Replacement due to bug ? https://github.com/dateutil/dateutil/issues/70#issuecomment-945080282
        try:
            # TODO better replacer using regex
            self.datetime = parser.parse(datetime_param.replace("UTC", "").replace(",", "").replace("MON", "").replace("TUES", "").replace("WED", "").replace("THURS", "").replace("FRI", "").replace("SAT", "").replace("SUN", ""))
        except parser.ParserError:
            print("Couldn't parse " + datetime_param)
            self.datetime = datetime.now()
        self.location = location
        self.url = url
        self.source: str = source
    
    # Thanks to https://stackoverflow.com/a/682545 & https://www.programiz.com/python-programming/methods/built-in/classmethod
    @classmethod
    def from_dict(cls, dict):
        return cls(
            name=dict["name"], 
            datetime_param=dict["datetime"],
            source=dict["source"],
            location=dict["location"],
            url=dict["url"]
            )
    
    
    def to_json(self):
        serializable_event = deepcopy(self)
        serializable_event.datetime = self.datetime.isoformat()
        return serializable_event.__dict__


    @property
    def clean_url(self):
        url = facebook_locale_to_www(self.url)
        if "?" in url:
            url = url[:url.index("?")]
        return url
    
    @property
    def description(self):
        return "Organized by {0}. See {1} for more info".format(self.source, self.clean_url)
    

    # Thanks to https://www.geeksforgeeks.org/getter-and-setter-in-python/
    @property
    def uid(self):
        # Facebook allows double entries of the same event, but in different times.
        # So the URL + datetime should be unique
        return slugify(self.clean_url) + slugify(self.datetime.isoformat())
    
    # See Add To Calendar Button documentation: https://github.com/add2cal/add-to-calendar-button#typical-structure
    @property
    def endTime(self):
        return self.datetime + timedelta(hours=2)
    
    """
    Okay so this is a doozy.
    - Glob is being used to future proof, in case of png or jpg or jpeg. That's good.
    - But it requires a local path to find the image
    - But that path cannot be relative. Originally this was passed img_dir=public/img,
      which then got public/ removed to img/filename.png.
      That works when running from the project dir! When run in C:/ProjectDir, it will
      look in C:/ProjectDir/public/img, find the thing, and replace as said above

      But the glob will fail when run from a different folder
      Cause then it will look for the file in C:/Differentdir/public/img and return None

    So this function got redesigned to not return ANY path, and instead just a filename.
    With optionally a relative path added as a paremeter

   
    """
    def get_image_filename(self, image_dir, return_with_path=None):
        try:
            image_glob_str = join(image_dir, self.uid + "*")
            filename_no_path = basename(glob(image_glob_str)[0])

            if not return_with_path:
                filename = filename_no_path
            else:
                filename = join(return_with_path, filename_no_path)

            return filename

        except IndexError:
            print("No image found for " + self.uid)
            return None
    
        
        

def load_events_from_json(json_path):
    events = []
    with open(json_path, "r") as file:
        raw_events = load(file)
        for raw_event in raw_events:
            event = Event.from_dict(raw_event)
            events.append(event)
    return events

def events_to_json(events, json_path):
    serializable_events = []
    for event in events:
        serializable_events.append(event.to_json())
    
    with open(json_path, "w") as file:
        dump(serializable_events, file)



# Built-in imports
from os import getenv
from os.path import join, realpath
from time import sleep
from json import loads
from urllib.request import urlretrieve

# Package imports
from selenium.webdriver.common.by import By
from selenium.common.exceptions import NoSuchElementException

# Local imports
from ..utils.fb_regexes import find_and_remove, regex_in, re_line_with_characters, re_guests, re_three_letter_two_digit_date, re_utc_time, re_utc_and_more
from ..Event import Event
from .driver import setup_driver
from ..utils.url_converter import facebook_www_to_locale
from .scrape_events_page import scrape_events_page

""" PARSING FUNCTIONS """


def scrape_all(driver, pages: list[str], img_dir: str):
    """ RUNNING PARSE FUNCTIONS ON PAGES """
    img_dir = realpath(img_dir)  # Directory where images get stored

    events = []
    for page in pages:
        driver.get(page)
        sleep(20)
        try:
            events.extend(scrape_events_page(driver, page, img_dir))


        except NoSuchElementException as e:
            print(f"Error parsing {page}")
            print("No events found.")
            print(e)

        except Exception as e:
            print(f"Error parsing {page}")
            raise e

    return events



from pathlib import Path
from datetime import datetime

from src.facebook_event_aggregator.Event import Event

test_event_one = Event(
    name="Underscores - Botanique",
    datetime_param=str(datetime.now()),
    source="Botanique",
    location="Brussels",
    url="https://www.facebook.com/events/3483289855332151"
)

test_event_two = Event(
    name="Chase Petra - Trix",
    datetime_param=str(datetime.now()),
    source="Trix",
    location="Antwerp",
    url="https://www.facebook.com/events/956264055785138/"
)

example = {
    "events": [test_event_one, test_event_two],
    "title": "Concerts",
    "host_domain": "https://concerts.example.com",
    
    "page_urls": ["https://www.facebook.com/botaniquebxl", "https://www.facebook.com/trixonline"],
    "timezone": "Europe/Brussels"
}


def check_event_contents_in(event: Event, haystack):
    for property in [event.name, event.description, event.location, event.clean_url]:
        assert property in haystack

def write_test_ics(path):
    with open(path.joinpath(str(path)), "w", encoding="UTF-8") as file:
        file.write(
            """
BEGIN:VCALENDAR
VERSION:2.0
PRODID:ics.py - http://git.io/RANDOM
BEGIN:VEVENT
CREATED:20151219T021727Z
DTEND;TZID=America/Toronto:20170515T110000
DTSTAMP:20151219T021727Z
DTSTART;TZID=America/Toronto:20170515T100000
LAST-MODIFIED:20151219T021727Z
RRULE:FREQ=DAILY;UNTIL=20170519T035959Z
SEQUENCE:0
SUMMARY:Meeting
TRANSP:OPAQUE
UID:21B97459-D97B-4B23-AF2A-E2759745C299
END:VEVENT
END:VCALENDAR
""")
        
        


# Built-in imports
from subprocess import run
from os.path import realpath, isdir, join, exists
from datetime import datetime

from git import Repo 

"""This module handles Git repo stuff"""

def _normalize_path(dir) -> str:
    return realpath(dir)

def _get_repo_path(parent_dir_path, repo_path) -> str:
    repo_path = _normalize_path(join(str(parent_dir_path), str(repo_path)))
    return repo_path

def _generate_commit_message():
    current_date = datetime.now().strftime("%d %m %Y")
    commit_msg = "Update from {}".format(current_date)
    return commit_msg


def clone_or_pull_repo(parent_dir_path, clone_dirname, repo_url: str):
    repo_path = _get_repo_path(parent_dir_path, clone_dirname)

    if not exists(repo_path):
        return Repo.clone_from(repo_url, repo_path)
    else:
        return Repo(repo_path).remote("origin").pull()

    
def update_repo(parent_dir_path, clone_dirname, commit_msg=None):
    repo_path = _get_repo_path(parent_dir_path, clone_dirname)
    
    if commit_msg is None:
        commit_msg = _generate_commit_message()
    
    repo = Repo(repo_path)
    
    repo.index.add(".")
    repo.index.commit(commit_msg)
    repo.remote("origin").push()



from os.path import exists, realpath
from pathlib import Path
from datetime import datetime

from src.facebook_event_aggregator.repo import _normalize_path, _get_repo_path, _generate_commit_message, clone_or_pull_repo, update_repo

from git import Repo, FetchInfo
from git.util import IterableList
from git.exc import GitCommandError

class TestRepo():    
    """Test helper funcs"""
    def test_normalize_path(self):
        assert _normalize_path("../") == realpath("../")
    
    def test_get_repo_path(self, tmp_path: Path):
        assert str(tmp_path.joinpath("test")) == _get_repo_path(tmp_path, "test")

    def test_generate_commit_message(self):
        assert _generate_commit_message() == f'Update from {datetime.now().strftime("%d %m %Y")}'


    def clone_or_pull_shortcut(self, tmp_path: Path):
        return clone_or_pull_repo(tmp_path, "test", "https://github.com/github/dev.git")

    def test_clone_or_pull(self, tmp_path: Path):
        repo_dir = _get_repo_path(tmp_path, "test")

        assert not exists(repo_dir)
        
        assert type(self.clone_or_pull_shortcut(tmp_path)) == Repo
        assert exists(repo_dir)

        assert type(self.clone_or_pull_shortcut(tmp_path)) == IterableList
        
        
    def test_update_repo(self, tmp_path: Path):
        repo_dir = _get_repo_path(tmp_path, "test")

        assert not exists(repo_dir)
        
        repo = self.clone_or_pull_shortcut(tmp_path)
        
        try:
            generated_commit_msg = _generate_commit_message()  # Do this before so there's less chance of bad timing failing the test
            update_repo(tmp_path, "test", commit_msg=None)
        except GitCommandError:
            pass  # This will return 403, which is normal
        finally:
            assert repo.commit(repo.branches[0]).message == generated_commit_msg
        
        try:
            update_repo(tmp_path, "test", commit_msg="Test succeeded!")
        except GitCommandError:
            pass  # This will return 403, which is normal
        finally:
            assert repo.commit(repo.branches[0]).message == "Test succeeded!"



from src.facebook_event_aggregator.utils.url_converter import facebook_locale_to_www, facebook_www_to_locale

class TestUrlConverter():
    www_url = "https://www.facebook.com/test"
    locale_url = "https://en-gb.facebook.com/test"


    def test_www_to_locale(self):
        assert facebook_www_to_locale(self.www_url, "en-gb") == self.locale_url

    def test_locale_to_www(self):
        assert facebook_locale_to_www(self.locale_url, "en-gb") == self.www_url
    


from src.facebook_event_aggregator.utils.fb_regexes import re_guests, re_line_with_characters, re_utc_and_more, re_three_letter_two_digit_date, re_utc_time, find_and_remove, regex_in

class TestFacebookRegexes():
    basic_string = "this is a string"
    complex_string = "Test! Meow!\nNOV\n11\nMore test!\nFri 19:00 UTC+01 AND 1 MORE\nName - 32 guests"
    
    def test_regex_in(self):
        assert regex_in(self.basic_string, r"[a-z]") == True
        assert regex_in(self.basic_string, r"[0-9]") == False
    

    def test_find_and_remove(self):
        remains, result = find_and_remove(self.basic_string, r"str.*ng")

        assert remains == "this is a "
        assert result == "string"

        remains, result = find_and_remove(self.basic_string, r"0.*9")

        assert remains == "this is a string"
        assert result == ""
    

    def test_re_line_with_characters(self):
        assert regex_in("Test", re_line_with_characters) == True
        assert regex_in("", re_line_with_characters) == False


    def test_re_utc_time(self):
        remains, result = find_and_remove(self.complex_string, re_utc_time)
        assert result == "Fri 19:00 UTC+01"


    def test_re_three_letter_two_digit_date(self):
        remains, result = find_and_remove(self.complex_string, re_three_letter_two_digit_date)
        assert result == "NOV\n11" 
        

    
    def test_re_guests(self):
        remains, result = find_and_remove(self.complex_string, re_guests)
        assert result == "Name - 32 guests"


    def test_re_utc_and_more(self):
        remains, result = find_and_remove(self.complex_string, re_utc_and_more)
        assert result == "UTC+01 AND 1 MORE"


from os import makedirs
from json import loads, dumps
from pathlib import Path

from dateutil import parser
from datetime import datetime


from src.facebook_event_aggregator.Event import Event, load_events_from_json, events_to_json
from src.facebook_event_aggregator.utils.url_converter import facebook_locale_to_www, facebook_www_to_locale

class TestEventClass():
    """Test helpers"""
    event = Event(
        name="Test",
        datetime_param="20 Nov 2023 18:00 UTC+1",
        source="Organisation",
        location="Belgium",
        url="https://en-gb.facebook.com/test"
    )

    def compare_to_original(self, event):
        assert event.name == "Test"
        assert event.datetime == parser.parse("20/11/2023 18:00+1")
        assert event.source == "Organisation"
        assert event.location == "Belgium"
        assert event.url == "https://en-gb.facebook.com/test"


    """Init tests"""
    def test_init(self):
        self.compare_to_original(self.event)

    def test_init_from_dict(self):
        event = Event.from_dict({
            "name":"Test",
            "datetime": "20 Nov 2023 18:00 UTC+1",
            "source": "Organisation",
            "location": "Belgium",
            "url": "https://en-gb.facebook.com/test"
        })
        assert event.to_json() == self.event.to_json()

    """Property tests"""
    def test_clean_url(self):
        assert self.event.clean_url == facebook_locale_to_www(self.event.url)
        assert self.event.clean_url != facebook_www_to_locale(self.event.url)


        test_event = Event("Name", "20 Nov 2023 18:00", "Source", "Location", "https://www.facebook.com/profile/?should_not_be_here")
        assert "should_not_be_here" not in test_event.clean_url
        assert "?" not in test_event.clean_url
        assert test_event.clean_url == "https://www.facebook.com/profile/"

    def test_description(self):
        assert type(self.event.description) == str
        assert self.event.source in self.event.description
        assert self.event.clean_url in self.event.description
    
    def test_uid(self):
        assert type(self.event.uid) == str
    
    def test_endTime(self):
        assert type(self.event.endTime) == datetime

    # TODO get_image

    """Method tests"""
    def test_to_json(self):
        self.compare_to_original(Event.from_dict(self.event.to_json()))
    
    def test_get_image_filename(self, tmp_path: Path):
        output_filename = self.event.uid + ".png"
        output_filepath = tmp_path.joinpath(output_filename)


        # does not exist
        assert self.event.get_image_filename(tmp_path) == None


        with open(output_filepath, "w") as file:
            file.write("")

        # return_with_path = None
        assert self.event.get_image_filename(tmp_path, return_with_path=None) == output_filename
        # return_with_path = defined
        assert self.event.get_image_filename(tmp_path, return_with_path=tmp_path) == str(output_filepath)
        


    """Global func tests"""
    def test_events_to_json_and_from_json(self, tmp_path: Path):
        output_path = str(tmp_path.joinpath("events.json"))
        
        events_to_json([self.event, self.event], output_path)
        events = load_events_from_json(output_path)

        for event in events:
            self.compare_to_original(event)

    
    #def test_events_from_json(self, tmp_path: Path):
     #   with open(tmp_path.joinpath("data.json"), "w", encoding="UTF-8") as file:
      #      file.write(f"{"",}")



from os import getenv

""" Helper functions for different facebook url's """

# To help get consistent results from scraping, a locale can be specified in .env
# This defaults to en-gb if none is provided
def facebook_www_to_locale(url, locale=getenv("fb_locale", "en-gb")):
    return url.replace("www.facebook", "{}.facebook".format(locale))

# And this function does the opposite, by removing the locale specification
# So that Facebook will detemine the language from the the end users' preference
def facebook_locale_to_www(url, locale=getenv("fb_locale", "en-gb")):
    return url.replace("{}.facebook".format(locale), "www.facebook")



# Built-in imports
from os import getenv
from time import sleep
from getpass import getpass

# Package imports
from selenium.webdriver.common.by import By

"""NOTE: currently unimplemented"""

def handle_fb_login(driver, headless):
    # If an email is provided, log into Facebook
    logged_in = False
    if getenv("facebook_email") is not None:
        facebook_email = getenv("facebook_email")
        print("Facebook email passed...")
        # Password provided, use that
        if getenv("facebook_password") is not None:
            print("... and a password. Using that to login.")
            facebook_password = getenv("facebook_password")
            login(driver, facebook_email, facebook_password)
            logged_in = True

        # No password provided, but running interactively, so prompt
        elif not headless:
            print("... but no password. Please enter it in the following prompt:")
            facebook_password = getpass()
            login(driver, facebook_email, facebook_password)
            logged_in = True
        
        # No password provided, but running headless, so quit
        else:
            print("Email was passed, but no password in headless mode. Skipping login")    
    
    return logged_in

def login(driver, email, password):
    driver.get("https://www.facebook.com/login")
    driver.find_element(By.ID, "email").send_keys(email)
    driver.find_element(By.ID, "pass").send_keys(password)
    driver.find_element(By.TAG_NAME, "form").submit()
    sleep(5)


# Built-in imports
from sys import argv
from os import makedirs
from os.path import realpath, dirname, join, exists
from json import load

# Package imports
from dateutil import parser
from ics import Calendar, Event as IcsEvent
from slugify import slugify

# Local imports
from ..repo import update_repo
from ..Event import load_events_from_json

def _read_ics_if_exists(ics_path):
    if exists(ics_path):
        with open(ics_path, "r", encoding="UTF-8") as file:
            ics_text = file.read()
        return Calendar(ics_text)
    else:
        return Calendar()

def events_to_ics(events: list, output_dir: str):
    output_dir = join(realpath(output_dir), "ical/")
    makedirs(output_dir, exist_ok=True)
    ics_all = join(output_dir, "all.ics")

    all_calendar = _read_ics_if_exists(ics_all)
    source_calendars = dict()

    sources = set([event.source for event in events])
    for source in sources:
        source_ics = join(output_dir, slugify(source) + ".ics")
        source_calendars[source] = _read_ics_if_exists(source_ics)
    

    for event in events:
        ics_event = IcsEvent()
        ics_event.name = event.name
        ics_event.begin = event.datetime
        # organizer also requires email
        #ics_event.organizer = event.organizer
        ics_event.location = event.location
        ics_event.url = event.url
        ics_event.uid = event.uid  # Thanks to uid, no duplicate entries will be made

        ics_event.description = event.description

        all_calendar.events.add(ics_event)
        source_calendars[event.source].events.add(ics_event)
        

    with open(ics_all, "w" , encoding="UTF-8") as file:
        file.writelines(all_calendar.serialize_iter())
    
    for source in source_calendars:
        dest = join(output_dir, slugify(source) + ".ics")
        calendar = source_calendars[source]

        with open(dest, "w", encoding="UTF-8") as file:
            file.writelines(calendar.serialize_iter())




# Built-in imports
from sys import argv, executable
from os import makedirs, getenv, execv, getcwd
from os.path import realpath, join, abspath, dirname
from pathlib import Path
from json import dump, load
from argparse import ArgumentParser

# Local imports
from .repo import clone_or_pull_repo, update_repo
from .Event import load_events_from_json, events_to_json
from .export import events_to_html, events_to_ics, cleanup_images

from .scraper import setup_driver, scrape_all


argparse = ArgumentParser()
argparse.add_argument("--target", "-t", nargs="+", help="Facebook page to extract from. Usage: --target https://www.facebook.com/trixonline/ --target https://www.facebook.com/botaniquebxl/")
argparse.add_argument("--update", "-u", action="store_true", help="Whether to push to Git")
argparse.add_argument("--scrape", "-s", action="store_true", help="Whether to run scraper (alternatively, load from cache)")

argparse.add_argument("--host-domain", "-hd", required=True, help="Domain where your files will be hosted. E.g. https://example.com")
argparse.add_argument("--timezone", "-tz", default="Europe/London", help="What timezone to use for ics events")
argparse.add_argument("--title", default="Title", help="What title to use for the html export")

argparse.add_argument("--repo", "--repo-url", required=True, help="Domain where your files will be hosted. E.g. https://example.com")



if __name__ == "__main__":
    args = argparse.parse_args()
    print(args)

    cwd = Path.cwd()
    export_dir = cwd.joinpath("public/")
    events_json_path = export_dir.joinpath("events.json") 
    img_dir = export_dir.joinpath("img/")
    


    #events_json = join(public_dir, "events.json")

    # Load .env file and startup params
    #startup_args = [arg.lower() for arg in argv[1:]]
    #headless = "headless" in startup_args
    #update = "update" in startup_args
    #scrape = "noscrape" not in startup_args
    scrape = args.scrape
    headless = True

    
    # Clone repo into public/
    clone_or_pull_repo(parent_dir_path=str(cwd), clone_dirname="public", repo_url=args.repo)
    makedirs(str(img_dir), exist_ok=True)

    if scrape:
        # Prepare img_dir in case it's needed

        # Setup Selenium scraper
        driver = setup_driver(headless)

        # Scrape events
        events = scrape_all(driver, args.target, str(img_dir))
    else:
        events = load_events_from_json(events_json_path)
                

    
    # Export
    if scrape:
        events_to_json(events, str(events_json_path))

    events_to_ics(events, str(export_dir))
    events_to_html(
        events=events,
        output_dir=str(export_dir),
        absolute_img_dir=str(img_dir),
        relative_from_html_img_dir="public/",
        host_domain=args.host_domain,
        page_urls=args.target,
        timezone=args.timezone,
        title=args.title)
    cleanup_images(events, str(img_dir))

    if args.update:
        update_repo(parent_dir_path=str(cwd), clone_dirname="public")

    # Cleanup
    if scrape:
        driver.quit()
            
